@@ -2,25 +2,19 @@
 #define _LIBHX_DEFS_H 1
 
 #ifdef __cplusplus
-#	define FIELD_SIZEOF(type, member) \
-		sizeof(static_cast<type *>(NULL)->member)
-#	define HXsizeof_member(type, member) FIELD_SIZEOF(type, member)
-#	define HXtypeof_member(type, member) \
-		__typeof__(static_cast<type *>(NULL)->member)
-#	if defined(__GNUC__) && __GNUC__ >= 4 && !defined(offsetof)
-		/*
-		 * This is here so most programs can skip inclusion
-		 * of stddef.h just to get offsetof.
-		 */
-#		define offsetof(type, member) __builtin_offsetof(type, member)
-#	endif
-#	ifndef offsetof
-#		define offsetof(type, member) \
-			reinterpret_cast<long>(&(static_cast<type *>(NULL)->member))
+#	define HXsizeof_member(type, member) sizeof(type::member)
+#	if __cplusplus >= 201100L
+#		define HXtypeof_member(type, member) decltype(type::member)
+#	else
+#		define HXtypeof_member(type, member) __typeof__(type::member))
 #	endif
 #	ifndef containerof
-#		define containerof(var, type, member) reinterpret_cast<type *>( \
-			reinterpret_cast<char *>(var) - offsetof(type, member))
+#		include <cstddef>
+#		include <type_traits>
+#		define containerof(var, T, member) reinterpret_cast<std::conditional<std::is_const<std::remove_pointer<decltype(var)>::type>::value, std::add_const<T>::type, T>::type *>(reinterpret_cast<std::conditional<std::is_const<std::remove_pointer<decltype(var)>::type>::value, const char, char>::type *>(var) - offsetof(T, member))
+#	endif
+#	ifndef static_cast
+#		define static_cast(T, x) static_cast<T>(x)
 #	endif
 
 template<typename new_type>
@@ -91,24 +85,33 @@ static __inline__ new_type signed_cast(unsigned char *expr)
 			((struct { type x; }){(expr)}.x)
 #	endif
 #	if defined(__GNUC__) && !defined(__clang__) && !defined(const_cast1)
-#		define __const_cast_strip1(expr) \
-			__typeof__(*(union { int z; __typeof__(expr) x; }){0}.x)
-#		define __const_cast_strip2(expr) \
-			__typeof__(**(union { int z; __typeof__(expr) x; }){0}.x)
-#		define __const_cast_strip3(expr) \
-			__typeof__(***(union { int z; __typeof__(expr) x; }){0}.x)
-#		define const_cast1(new_type, expr) ({ \
-			BUILD_BUG_ON(!__builtin_types_compatible_p(__const_cast_strip1(expr), __const_cast_strip1(new_type))); \
-			(new_type)(expr); \
-		})
-#		define const_cast2(new_type, expr) ({ \
-			BUILD_BUG_ON(!__builtin_types_compatible_p(__const_cast_strip2(expr), __const_cast_strip2(new_type))); \
-			(new_type)(expr); \
-		})
-#		define const_cast3(new_type, expr) ({ \
-			BUILD_BUG_ON(!__builtin_types_compatible_p(__const_cast_strip3(expr), __const_cast_strip3(new_type))); \
-			(new_type)(expr); \
-		})
+		/*
+		 * The idea starts with (in abstract notation)
+		 * 	typeof deref typeof expr
+		 * To deref something, we need an object, which we can get by
+		 * creating a temporary aggregate, such as a union, of which
+		 * the member is accessed and dereferenced.
+		 * 	*(union { __typeof__(expr) x; }){init}.x
+		 * union has two nice properties:
+		 * - with an additional dummy member, we do not have to
+		 *   initialize x according to its type, which, if expr is
+		 *   an array type, may want extra braces.
+		 * - and with that dummy member, we also avoid the ugly
+		 *   "literal 0 is implicitly convertible to a pointer".
+		 * Unfortunately, this all requires C99 compound initializers.
+		 * That's ok - gcc and clang only treat it as a warning even
+		 * under strict C89 - and if you still force strict C89 on
+		 * yourself, you have a lot to answer for either way.
+		 */
+#		define __const_cast_strip(ptrs, expr) \
+			__typeof__(ptrs(union { int z; __typeof__(expr) x; }){0}.x)
+#		define __const_cast_p(ptrs, new_type, expr) ((new_type)( \
+			(expr) + \
+			BUILD_BUG_ON_EXPR(!__builtin_types_compatible_p(__const_cast_strip(ptrs, expr), __const_cast_strip(ptrs, new_type))) \
+		))
+#		define const_cast1(new_type, expr) __const_cast_p(*, new_type, expr)
+#		define const_cast2(new_type, expr) __const_cast_p(**, new_type, expr)
+#		define const_cast3(new_type, expr) __const_cast_p(***, new_type, expr)
 #	endif
 #	ifndef signed_cast
 #		define signed_cast(type, expr)      ((type)(expr))
@@ -127,14 +130,8 @@ static __inline__ new_type signed_cast(unsigned char *expr)
 #	ifndef reinterpret_cast
 #		define reinterpret_cast(type, expr) ((type)(expr))
 #	endif
-#	if defined(__GNUC__) && __GNUC__ >= 4 && !defined(offsetof)
-#		define offsetof(type, member) __builtin_offsetof(type, member)
-#	endif
-#	ifndef offsetof
-#		define offsetof(type, member) \
-			reinterpret_cast(long, &(static_cast(type *, NULL)->member))
-#	endif
 #	ifndef containerof
+#		include <stddef.h>
 #		define containerof(var, type, member) reinterpret_cast(type *, \
 			reinterpret_cast(char *, var) - offsetof(type, member))
 #	endif
@@ -171,6 +168,25 @@ static __inline__ new_type signed_cast(unsigned char *expr)
 #ifndef O_BINARY
 #	define O_BINARY 0
 #endif
+#ifndef S_IRGRP
+	/* Can happen in mingw */
+#	define S_IRGRP (S_IRUSR >> 3)
+#endif
+#ifndef S_IWGRP
+#	define S_IWGRP (S_IWUSR >> 3)
+#endif
+#ifndef S_IXGRP
+#	define S_IXGRP (S_IXUSR >> 3)
+#endif
+#ifndef S_IROTH
+#	define S_IROTH (S_IRUSR >> 6)
+#endif
+#ifndef S_IWOTH
+#	define S_IWOTH (S_IWUSR >> 6)
+#endif
+#ifndef S_IXOTH
+#	define S_IXOTH (S_IXUSR >> 6)
+#endif
 #ifndef S_IRUGO
 #	define S_IRUGO (S_IRUSR | S_IRGRP | S_IROTH)
 #endif
@@ -193,4 +209,19 @@ static __inline__ new_type signed_cast(unsigned char *expr)
 #define __HX_STRINGIFY_EXPAND(s) #s
 #define HX_STRINGIFY(s) __HX_STRINGIFY_EXPAND(s)
 
+#ifndef container_of
+#	define container_of(v, s, m) containerof((v), s, m)
+#endif
+
+#if !defined(_WIN32) || (defined(__USE_MINGW_ANSI_STDIO) && __USE_MINGW_ANSI_STDIO + 0 > 0)
+	/*
+	 * Sufficiently old versions of the VC runtime do not even support %ll.
+	 */
+#	define HX_LONGLONG_FMT "ll"
+#	define HX_SIZET_FMT "z"
+#else
+#	define HX_LONGLONG_FMT "I64"
+#	define HX_SIZET_FMT "I"
+#endif
+
 #endif /* _LIBHX_DEFS_H */
