@@ -10,6 +10,7 @@
 #define LIBHX_INTERNAL_H 1
 
 #include "config.h"
+#include <stdint.h>
 #include <libHX/defs.h>
 #include <libHX/string.h>
 
@@ -42,6 +43,9 @@
 #define MAXLNLEN 1024 /* max length for usual line */
 
 #define HXMC_IDENT 0x200571AF
+#if !defined(__cplusplus)
+#	define nullptr NULL
+#endif
 
 struct memcont {
 	size_t alloc, length;
@@ -53,5 +57,6 @@ struct timespec;
 struct timeval;
 
 extern hxmc_t *HXparse_dequote_fmt(const char *, const char *, const char **);
+extern size_t HX_substr_helper(size_t, long, long, size_t *);
 
 #endif /* LIBHX_INTERNAL_H */
