@@ -1,10 +1,4 @@
-/*
- *	Copyright Jan Engelhardt
- *
- *	This program is free software; you can redistribute it and/or
- *	modify it under the terms of the WTF Public License version 2 or
- *	(at your option) any later version.
- */
+// SPDX-License-Identifier: MIT
 #ifndef __cplusplus
 #	include <stdlib.h>
 #else
@@ -38,7 +32,8 @@ int main(int argc, const char **argv)
 		if (stat(argv[1], &sa) < 0 ||
 		    stat(argv[2], &sb) < 0)
 			perror("stat");
-		printf("Difference: %ld\n", HX_time_compare(&sa, &sb, 'm'));
+		else
+			printf("Difference: %ld\n", HX_time_compare(&sa, &sb, 'm'));
 	}
 
 	HX_exit();
