@@ -1,10 +1,7 @@
+// SPDX-License-Identifier: MIT
+// SPDX-FileCopyrightText: 2022 Jan Engelhardt
 /*
  *	option parser test program
- *	Copyright Jan Engelhardt
- *
- *	This program is free software; you can redistribute it and/or
- *	modify it under the terms of the WTF Public License version 2 or
- *	(at your option) any later version.
  */
 #include <errno.h>
 #include <stdio.h>
