@@ -1,13 +1,9 @@
 /*
 A=b;C="d" ; E="F;" ; F= G=Z
 */
+// SPDX-License-Identifier: MIT
 /*
  *	shconfig test program
- *	Copyright Jan Engelhardt
- *
- *	This program is free software; you can redistribute it and/or
- *	modify it under the terms of the WTF Public License version 2 or
- *	(at your option) any later version.
  */
 #include <errno.h>
 #include <stdio.h>
